#!/usr/bin/env python
#-*- coding: ISO-8859-1 -*-
#pylint: disable-msg=E1101,C0103,R0902,R0903,C0301,R0914,R0913,R0915,W0613,W0612,W0704,W0107
#Author: Valentin Kuznetsov

"""
FileMover web-server
"""

import os
import sys
import stat
import time
import urllib
import traceback
import cherrypy
from   cherrypy import expose

import ConfigParser
from   FMWSConfig  import FMWSConfig
from   FMWSLogger  import FMWSLogger, setLogger
from   FileManager import Server, validate_lfn
from   Status import StatusMsg, StatusCode
from   DBSInteraction import DBS
from   Utils import sizeFormat, getArg, sendEmail, getExcMessage
from   Utils import cleanup, iparser
from   CMSRunManager import CMSRunThreadWrapper

try:
    from RequestServer import RequestMgr
except:
    pass

from   Cheetah.Template import Template
from   Templates import templateTop, templateForm

# webtools framework
from  Framework import Controller
#from  Framework import Context

from  Tools.SecurityModuleCore import decryptCookie
from  Tools.SecurityModuleCore import SecurityDBApi
from  Tools.SecurityModuleCore.SecurityDBApi import SecurityDBApi

def ajaxResponse(msg, tag="_response", element="object"):
    """AJAX form wrapper"""
    page  = """<ajax-response><response type="%s" id="%s">""" % (element, tag)
    if  element == 'element':
        page += msg
    else:
        page += "<div class=\"normal\">%s</div>" % msg
    page += "</response></ajax-response>"
    return page

def getBottomHTML():
    """HTML bottom template"""
    page = "</body></html>"
    return page

def inputError(msg, dataset, eventset, eventlist, email, site):
    """pick-event input error handler"""
    page  = msg
    page += '<br/>\nProvided input:\n'
    page += '<pre>\n'
    page += 'dataset:' + dataset + '\n'
    page += 'eventset:\n' + eventset + '\n'
    page += '</pre>\n'
    if  eventlist:
        page += 'DBS info about this input:<br/>\n'
        page += '<ul>\n'
        query = 'find file where dataset=%s and site like %s ' % (dataset, site)
        for run, event, lumi in eventlist:
            uinput = query+' and run=%s and lumi=%s' % (run, lumi)
            link   = '<a href="https://cmsweb.cern.ch/dbs_discovery/aSearch?caseSensitive=on&amp;userMode=user&amp;sortOrder=desc&amp;sortName=&amp;grid=0&amp;method=dbsapi&amp;dbsInst=cms_dbs_prod_global&amp;userInput=%s">here</a>' % urllib.quote(uinput)
            page += '<li>click \n%s\n for run=%s evt=%s lumi=%s</li>\n' % (link, run, event, lumi)
        page += '</ul>\n'
    if  email:
        msg   = page.replace('<pre>','').replace('</pre>','') 
        msg   = msg.replace('<ul>','').replace('</ul>','').replace('<br/>','')
        msg   = msg.replace('</li>','').replace('<li>','')
        msg   = msg.replace('<a href=','').replace('>here</a>','')
        sendEmail(email, msg, 'not completed')
    page += '<script type="text/javascript">EFormAction("")</script>'
    page  = ajaxResponse(page)
    return page


def spanId(lfn):
    """assign id for span tag based on provided lfn"""
    return lfn.split("/")[-1].replace(".root","")

def formatLFNList(user, iList, sList):
    """HTML formatted list of LNFs"""
    page  = ""
    style = ""
    for idx in xrange(0, len(iList)):
        lfn  = iList[idx]
        fstat = sList[idx]
        if  style:
            style = ""
        else:
            style = "class=\"zebra\""
        spanid = spanId(lfn)
        page  += """<tr %s><td class="td_left">
%s
</td><td align="right">
<span id="%s">%s</span>
</td></tr>
<script type="text/javascript">
var lfnUpdater = new Updater('%s');
ajaxEngine.registerAjaxObject('%s',lfnUpdater);
</script>
""" % (style, lfn, spanid, fstat, spanid, spanid)
        if  fstat:
            page += """<script type="text/javascript">setTimeout('ajaxStatusOne(\\'%s\\',\\'%s\\')',1000)</script>""" % (user, lfn)
    return page
    
class FMWS(FMWSLogger, Controller):
    """FileMover web-server based on CherryPy"""
    def __init__(self, context=None):
        self.dbs = DBS()
        self.securityApi    = ""
        self.fmwsdir        = os.getcwd() # remember location where we started
        self.fmConfig       = FMWSConfig()
        self.download       = self.fmConfig.download_area()
        try:
            if  context:
                context.OptionParser ().add_option("-v", "--verbose", 
                        action="store",type="int", default=0, 
                        dest="verbose",
                        help="specify verbosity level, 0-none, 1-info, 2-debug")
                self.securityApi = SecurityDBApi(context)
            Controller.__init__ (self, context, __file__)
            self.config = self.setConfig(base="FMWS")
        except:
            traceback.print_exc()
            pass
        Server.configure(self.fmConfig.config)
        self.verbose        = self.fmConfig.verboseLevel()
        self.voms_timer     = 0
        self.userDict       = {}
        self.userDictPerDay = {}
        self.baseUrl        = ""
        self.url            = "/filemover"
#        self.host           = self.fmConfig.host()

        # configure cmsRun manager
        cp = ConfigParser.ConfigParser()
        cp.add_section("release_manager")
        cp.set("release_manager", "max_cmsrun", "2")
        self.cmsrun = CMSRunThreadWrapper(self.fmConfig.cmsswDir())
        self.cmsrun.configure(cp)
        
        FMWSLogger.__init__(self, self.fmConfig.loggerDir(), "FMWSServer", self.verbose)
        setLogger('cherrypy.access', super(FMWS, self).getHandler(), super(FMWS, self).getLogLevel())
        setLogger('cherrypy.error', super(FMWS, self).getHandler(), super(FMWS, self).getLogLevel())
        setLogger('ThreadPool', super(FMWS, self).getHandler(), super(FMWS, self).getLogLevel())
        setLogger('TransferWrapper', super(FMWS, self).getHandler(), super(FMWS, self).getLogLevel())
        setLogger('FileLookup', super(FMWS, self).getHandler(), super(FMWS, self).getLogLevel())

        try:
            self.requestmgr = RequestMgr(dbhost=self.fmConfig.dbhost(),
                                         dbport=self.fmConfig.dbport(),
                                         limit=self.fmConfig.dayTransfer())
        except Exception, ex:
            pass

    @expose
    def allow(self, userdn, **kwargs):
        """Check if provided DN is allowed to make more file mover requtest"""
        res = False
        try:
            res = self.requestmgr.allow(userdn)
        except:
            pass
        return res

    @expose
    def record(self, **kwargs):
        """
        Record provided src/dst/date into filemover.collection DB
        """
        try:
            src  = kwargs.get('src')
            dst  = kwargs.get('dst')
            date = kwargs.get('date')
            udn  = kwargs.get('userdn')
            return self.requestmgr.record(udn, src, dst, date)
        except:
            traceback.print_exc()
            return "fail"

    def readyToRun(self):
        """WEBTOOLS handler to setup run-time parameters"""
        opts = self.context.CmdLineArgs().opts
        self.baseUrl = opts.baseUrl
        self.url = "/" + self.baseUrl.split("/")[-1]
        self.verbose = opts.verbose

    def handleExc(self):
        """exception handler"""
        msg = getExcMessage()
        self.writeLog(traceback.format_exc())
        page  = "<div>Request failed</div>"
        page += "<pre>%s</pre>" % msg
        return page

    @expose
    def unlock(self, **kwargs):
        """default service method"""
        cherrypy.response.headers['Content-Type'] = 'text/xml'
        userName = self.getUserName()
        if  not userName or userName == "guest" or userName.lower() == "unknown":
            page = "In order to use CMS File service you must login with your HyperNews login name and password.</p><p>Please follow Login link in right upper corner of the page."
        else:
            userName = userName.replace(".", "")
            pwdfile = os.path.join(os.environ['HOME'], '.xrd/genpwd/%s' % userName)
            if  not os.path.isfile(pwdfile):
                try:
                    os.system('yes | xrdpwdadmin add %s' % userName)
                except:
                    pass
            if  os.path.isfile(pwdfile):
                cmd = 'cat ~/.xrd/genpwd/%s | egrep "^password:"' % userName
                pwd = os.popen(cmd).read().split()[-1]
            else:
                pwd = 'N/A'
            page = '<span class="box_pwd">xrootd password: %s</span>' % pwd
        page = ajaxResponse(page, tag='xrootd', element='element')
        return page

    @expose
    def index(self, user = "USER"):
        """default service method"""
        page = self.getTopHTML()
        userName = self.getUserName()
        if  not userName or userName == "guest" or userName.lower() == "unknown":
            page += "<p>In order to use CMS File service you must login with your HyperNews login name and password.</p><p>Please follow Login link in right upper corner of the page.</p>"
        else:
            user  = userName
            self.addUser(user)
            page += self.userForm(user)
        page += getBottomHTML()
        return page

    def addUser(self, user):
        """add user to local cache"""
        if  not self.userDict.has_key(user):
            self.userDict[user] = ([], [])
            try:
                os.makedirs("%s/download/%s/softlinks" % (self.download, user))
            except:
                pass

    def getTopHTML(self):
        """HTML top template"""
        nameSpace = {'url': self.url}
        page = templateTop(searchList=[nameSpace]).respond()
        return page

    def getStat(self, user):
        """get status of current user's job"""
        try:
            iList, sList = self.userDict[user]
            iList.reverse() # reverse list to see newly requested files first
            sList.reverse()
        except:
            pass
            return ""
        page = """
<br />
<div>
<table class="fmws_table">
%s
</table>
</div>
        """ % formatLFNList(user, iList, sList)
        return page
        
    def getStatForLfn(self, user, lfn):
        """return status of user requested lfn"""
        validate_lfn(lfn)
        page  = ""
        lfnList, statList = self.userDict[user]
        try:
            idx    = lfnList.index(lfn)
            fstat  = statList[idx]
            spanid = spanId(lfn)
            page  += """%s""" % fstat
            if  fstat.find("Download") == -1:
                page +=" | <a href=\"javascript:ajaxCancel('%s','%s')\">Cancel</a> " % (user, lfn)
        except:
            traceback.print_exc()
            print lfn
            print self.userDict
            pass
        return page
        
    def userForm(self, user):
        """page forms"""
        nameSpace = {'user':user, 'cache': self.checkUserCache(user)}
        page = templateForm(searchList=[nameSpace]).respond()
        return page

    @expose
    def getEvent(self, user, dataset, eventset, email, **kwargs):
        """pick-event method"""
        cherrypy.response.headers['Content-Type'] = 'text/xml'
        # generate requestdict to be used by self.cmsrun
	msg  = ""
        site = self.fmConfig.cmsswSite()
        try:
            eventlist = [x for x in iparser(eventset)]
        except:
            msg  = "Fail to parse event set. It should be in a form:<br />"
            msg += "<pre>run event lumi\nrun event lumi\nrun event lumi\n.......</pre>"
            return inputError(msg, dataset, eventset, '', email, site)

        requestdict = { 'dataset' : dataset, 'eventlist' : eventlist,
                        'eventset' : eventset, 'email' : email, 'site': site} 
        
        if  not dataset or not eventset:
            msg = "You MUST specify all input fields"
            return inputError(msg, dataset, eventset, eventlist, email, site)

        if  len(eventlist) > 20:
            msg = "Too many run/event/lumi sets, current limit is 20."
            return inputError(msg, dataset, eventset, eventlist, email, site)

        page = getArg(kwargs, 'page', '')
        try:
            requestId = self.cmsrun.submit(user, requestdict)
        except Exception, e:
	    traceback.print_exc()
            msg = sys.exc_info()[1]
            return inputError(str(msg), dataset, eventset, eventlist, email, site)

        if  requestId:
            page += "EDM config file has been generated and request submitted to run cmsRun job."
            page += '<br/><img src="images/loading.gif" /> please wait ...'
            page +="""<script type="text/javascript">setTimeout("ajaxcmsRunStatus(\'%s\',\'%s\')",5000)</script>""" % (user, requestId)
        else:
            page += 'We are sorry, server is busy with other cmsrun jobs, please try again later'
            page += '<script type="text/javascript">EFormAction("")</script>'
        page = ajaxResponse(page)
        return page

    @expose
    def cmsRunStatus(self, user, requestid, **kwargs):
        """return status of cmsRun job"""
        cherrypy.response.headers['Content-Type'] = 'text/xml'
        page = '<img src="images/loading.gif" /> '
        try:
            status = self.cmsrun.status(requestid)
        except:
            traceback.print_exc()
            status = 1 # error status see below
        if  status == 'busy':
            page  = 'sorry, server is busy with other cmsrun jobs, please try again later'
            page += '<script type="text/javascript">EFormAction("")</script>'
            page  = ajaxResponse(page)
            return page
            
        outfilename = self.cmsrun.getoutfile(requestid)
        if  status and status > 0:
            try:
                page  = "Sorry, we cannot fetch your event, cmsRun job failed<br/>"
                page += "Process output:"
                page += "<pre>%s</pre>\n" % self.cmsrun.process_output(requestid)
                page += "CmsRun configuration:"
                page += "<pre>%s</pre>\n" % self.cmsrun.config(requestid)
                page += '<script type="text/javascript">EFormAction("")</script>'
                page  = ajaxResponse(page)
                self.cmsrun.clean(requestid)
            except:
                traceback.print_exc()
                page  = 'Unable to get cmsrun status, reason \n'
                page += traceback.format_exc()
            return page
            
        if  status == 0:
            try:
                fileStat = os.stat(outfilename)
                fileSize = sizeFormat(fileStat[stat.ST_SIZE])
                oname = outfilename.split("/")[-1]
                ofile = "%s/download/%s/softlinks/%s" % (self.download, user, oname)
                try:
                    os.remove(ofile)
                except:
                    pass
                os.symlink(outfilename, ofile)
                link  = "download/%s/softlinks/%s" % (user, oname)
                # demangle parameters from oname
                requestdict = self.cmsrun.inputparams(requestid)
                page  = 'Request sucessfully completed for\n'
                page += '<pre>\n'
                page += 'dataset : %s\n' % requestdict['dataset']
                page += 'eventset:\n%s\n' % requestdict['eventset']
                page += '</pre>\n'
                msg   = page.replace('<pre>','').replace('</pre>','')
                nevts = self.cmsrun.nevents(requestid)
                if  nevts:
                    page += 'Number of events: %s\n' % nevts
                    msg  += 'Number of events: %s\n' % nevts
                else:
                    page += '<span class="red">No events were written out</span>\n'
                    msg  += 'No events were written out\n'
                page += '<br/>File <a href="%s">download (%s)</a> ' % (link, fileSize)
                url   = self.url+'/'+link
                msg  += 'link    : %s\n' % url
                msg  += 'size    : %s\n' % fileSize
                sendEmail(requestdict['email'], msg, requestid)
                page += """| <a href=\"javascript:ajaxrm('%s','%s')\">remove</a>""" \
                    % (user, link)
                page += "<br/><br/>Used release: %s" \
                            % self.cmsrun.release_to_use(requestid)
                page += """<br/>Used config: 
                    <a href="javascript:ShowTag('config')">show</a> |
                    <a href="javascript:HideTag('config')">hide</a>"""
                page += "<div id=\"config\" class=\"hide\"><pre>%s</pre></div>"\
                            % self.cmsrun.config(requestid)
                page += """<br/>Output of edmFileUtil (file content):
                    <a href="javascript:ShowTag('edm')">show</a> |
                    <a href="javascript:HideTag('edm')">hide</a>"""
                content = self.cmsrun.filecontent(requestid)
                content = content.replace('file:%s' % outfilename, '')
                content = content.replace(outfilename, '')
                content = content.replace('<', '&lt;')
                content = content.replace('>', '&gt;')
                page += "<div id=\"edm\" class=\"hide\"><pre> %s </pre></div>\n" \
                            % content
                self.cmsrun.clean(requestid)
                page += '<script type="text/javascript">EFormAction("")</script>'
                page  = ajaxResponse(page)
            except:
                traceback.print_exc()
                page  = 'Unable to get cmsrun status, reason \n'
                page += traceback.format_exc()
            return page
        if  os.path.isfile(outfilename):
            page += 'cmsRun wrote %s bytes, still working on it ...\n' \
                % os.stat(outfilename)[stat.ST_SIZE]
        else:
            page += 'cmsRun job is launched, please wait\n'
        page += """<script type="text/javascript">setTimeout("ajaxcmsRunStatus(\'%s\', \'%s\')",5000)</script>""" % (user, requestid)
        page = ajaxResponse(page)
        return page
        
    @expose
    def resolveLfn(self, user, dataset, run, minEvt, maxEvt, branch, **kwargs):
        """look-up LFNs in DBS upon user request"""
        cherrypy.response.headers['Content-Type'] = 'text/xml'
        if  not minEvt:
            evt = ""
        elif not maxEvt:
            evt = minEvt
        else:
            evt = (minEvt, maxEvt)
        lfnList = self.dbs.getFiles(run, dataset, evt, branch)
        if  not lfnList:
            page = "No lfns found for your criteria"
        else:
            page  = """Found the following list of lfns:<br/>\n"""
            page += """<table class="normal">\n"""
            for lfn, size in lfnList:
                page += "<tr>\n"
                page += """<td>%s</td><td>%s</td><td><a href="javascript:ajaxRequest('%s','%s')">request</a></td>\n""" % (lfn, size, user, lfn)
                page += "</tr>\n"
            page += "</table>\n"
        page += """<script type="text/javascript">clearInterval()</script>"""
        page  = ajaxResponse(page)
        return page
        
    def nActiveDownloads(self, user):
        """check number of active downloads/user"""
        lfnList, statList = self.userDict[user]
        count = 0
        for istat in statList:
            if  istat.find("Download") == -1:
                count += 1
        return  count
        
    def addLfn(self, user, lfn, **kwargs):
        """add LFN request to the queue"""
        validate_lfn(lfn)
        # check how may requests in total user placed today
        today = time.strftime("%Y%m%d", time.gmtime(time.time()))
        if  self.userDictPerDay.has_key(user):
            nReq, day = self.userDictPerDay[user]
            if  day != today and nReq > self.fmConfig.dayTransfer():
                return 0
            else:
                if  day != today:
                    self.userDictPerDay[user] = (0, today)
        else:
            self.userDictPerDay[user] = (0, today)
        lfnList, statList = self.userDict[user]
        # check how may requests user placed at once
        if  len(lfnList) < int(self.fmConfig.maxTransfer()):
            if  not lfnList.count(lfn):
                lfnList.append(lfn)
                statList.append('requested')
                nRequests, day = self.userDictPerDay[user]
                self.userDictPerDay[user] = (nRequests+1, today)
            else:
                return 2
        else: 
            return 0
        self.userDict[user] = (lfnList, statList)
        return 1
        
    def delLfn(self, user, lfn):
        """delete LFN from the queue"""
        validate_lfn(lfn)
        lfnList, statList = self.userDict[user]
        if  lfnList.count(lfn):
            idx = lfnList.index(lfn)
            lfnList.remove(lfn)
            statList.remove(statList[idx])
            self._remove(user, lfn)

    def checkUserCache(self, user):
        """check users's cache"""
        page = ""
        lfnList, statList = self.userDict[user]
        pfnList = os.listdir("%s/download/%s/softlinks" % (self.download, user))
        for ifile in pfnList:
            f = "%s/download/%s/softlinks/%s" % (self.download, user, ifile)
            abspath = os.readlink(f)
            if  not os.path.isfile(abspath):
                # we got orphan symlink
                try:
                    os.remove(f)
                except:
                    pass
                continue
            fileStat = os.stat(abspath)
            fileSize = sizeFormat(fileStat[stat.ST_SIZE])
            link     = "download/%s/%s" % (user, ifile)
            lfn      = abspath.replace(self.fmConfig.transferDir(),"")
            msg = "<a href=\"%s/%s\">Download (%s)</a> | <a href=\"javascript:ajaxRemove('%s','%s')\">Remove</a> " % (self.url, link, fileSize, user, lfn)
            # skip files who hold events and clean them up if they stay too long
            if  f.find('events_dataset__') != -1:
                cleanup(f)
                cleanup(abspath)
            else:
                if  not lfnList.count(lfn):
                    lfnList.append("%s" % lfn)
                    statList.append(msg)
        self.userDict[user] = (lfnList, statList)
        page += self.getStat(user)
        return page
        
    def setStat(self, user, _lfn=None):
        """update status of retrieving LFN"""
        statCode = -1
        if  not self.userDict.has_key(user):
            self.userDict[user] = ([], [])
        lfnList, statList = self.userDict[user]
        for idx in xrange(0, len(lfnList)):
            lfn    = lfnList[idx]
            if  _lfn and lfn != _lfn:
                continue
            prevStat = statList[idx]
            status = Server.status(lfn)
            if  status == "This LFN has not been requested yet!":
                status = (0, 'orphan')
	    elif status == 'Unknown status; internal error.':
                status = (0, 'orphan')
            if  lfn.find("/") == -1:
                statCode = 0
                statMsg  = prevStat 
                statList[idx] = statMsg
                ifile = lfn
                pfn = "%s/download/%s/%s" % (self.download, user, ifile)
            else:
                statCode = status[0]
                statMsg  = status[1]
                statList[idx] = statMsg
                iList  = lfn.split("/")
                ifile  = iList[-1]
                idir   = self.fmConfig.transferDir() + '/'.join(iList[:-1])
                pfn    = os.path.join(idir, ifile)
            if  statCode == 0:
                if  os.path.isfile(pfn):
                    if  not os.path.isfile("%s/download/%s/%s" % (self.download, user, ifile)):
			try:
                            os.link(pfn, "%s/download/%s/%s" % (self.download, user, ifile))
                            os.symlink(pfn, "%s/download/%s/softlinks/%s" % (self.download, user, ifile))
			except:
			    traceback.print_exc()
			    pass
                    link     = "download/%s/%s" % (user, ifile)
                    filepath = "%s/download/%s/%s" % (self.download, user, ifile)
                    fileStat = os.stat(filepath)
                    fileSize = sizeFormat(fileStat[stat.ST_SIZE])
                    lfn      = pfn.replace(self.fmConfig.transferDir(), "")
                    msg = "<a href=\"%s/%s\">Download (%s)</a> | <a href=\"javascript:ajaxRemove('%s','%s')\">Remove</a> " % (self.url, link, fileSize, user, lfn)
                    statList[idx] = msg
                else:
                    statList[idx] = "Pick event or orphan file, <a href=\"javascript:ajaxRemove('%s','%s')\">Remove</a> " % (user, lfn)
        self.userDict[user] = (lfnList, statList)
        return statCode

    def getUserName(self):
        """
           Get userName from stored cookie, should be run within WEBTOOLS framework
        """
        userName = ""
        header = cherrypy.request.headers
	#print "HEADER", header
        if  header.has_key('Ssl-Client-S-Dn'):
            dn = header['Ssl-Client-S-Dn']
            try:
                userName = self.securityApi.getUsernameFromDN(dn)[0]['username']
            except:
#                traceback.print_exc()
                pass
            if  userName:
                return userName
        try:
            cookie = cherrypy.request.cookie
            userName = decryptCookie (cookie["dn"].value, self.securityApi)
        except:
            traceback.print_exc()
            pass
        return userName

    def _remove(self, user, lfn):
        """remove requested LFN from the queue"""
        validate_lfn(lfn)
        ifile = lfn.split("/")[-1]
        # remove soft-link from user download area
        try:
            link = "%s/download/%s/softlinks/%s" % (self.download, user, ifile)
            os.unlink(link)
        except:
            pass
        # remove hard-link from user download area
        try:
            link = "%s/download/%s/%s" % (self.download, user, ifile)
            os.unlink(link)
        except:
            pass
        # now time to check if no-one else has a hardlink to pfn, if so remove pfn
        try:
            pfn = self.fmConfig.transferDir() + lfn
            fstat = os.stat(pfn)
            if  int(fstat[stat.ST_NLINK]) == 1: # only 1 file exists and no other hard-links to it
                os.remove(pfn)
        except:
            pass

    @expose
    def remove(self, user, lfn, **kwargs):
        """remove requested LFN from the queue"""
        validate_lfn(lfn)
        cherrypy.response.headers['Content-Type'] = 'text/xml'
        remove = getArg(kwargs, 'remove', 0)
        page = ""
        self._remove(user, lfn)
        if  not remove:
            self.delLfn(user, lfn)
            page += self.getStat(user)
        return ajaxResponse(page)

    def tooManyRequests(self, user):
        """report that user has too many requests"""
        page = "<p>Too many requests, you're allowed to fetch only %s LFNs at once and place only %s per day</p>" % (self.fmConfig.maxTransfer(), self.fmConfig.dayTransfer())
        if  self.userDictPerDay.has_key(user):
            today = time.strftime("%Y%m%d", time.gmtime(time.time()))
            nReq, day = self.userDictPerDay[user]
            if  day != today and nReq > self.fmConfig.dayTransfer():
                page = "<p>You are reached your request/day limit, this service allows to place only %s requests/per user/per day. If you need to transfer a large amount of data please consider using <a href=\"http://cmsweb.cern.ch/phedex/\">PhEDEx</a> service</p>" % self.fmConfig.dayTransfer()
        page += self.getStat(user)
        return ajaxResponse(page)

    @expose
    def request(self, user, lfn, **kwargs):
        """place LFN request"""
        validate_lfn(lfn)
        cherrypy.response.headers['Content-Type'] = 'text/xml'
        page = getArg(kwargs, 'page', '')
        lfn = lfn.strip() # remove spaces around lfn
        lfnStatus = self.addLfn(user, lfn)
        if  not lfnStatus:
            return self.tooManyRequests(user)
        lfn = urllib.unquote(lfn)
        page = ""
        try:
            if  lfnStatus == 1:
                Server.request(lfn)
                page += "<div>Requested file:<p><em>%s</em></p> has been placed into the transfer queue</div>" % lfn
            else:
                page += "<div>Requested file:<p><em>%s</em></p> is already in a transfer queue</div>" % lfn
            page += self.getStat(user)
        except:
            page = self.handleExc()
            pass
        page = ajaxResponse(page)
        return page
        
    @expose
    def cancel(self, user, lfn, **kwargs):
        """cancel LFN request"""
        validate_lfn(lfn)
        cherrypy.response.headers['Content-Type'] = 'text/xml'
        self.delLfn(user, lfn)
        lfn = urllib.unquote(lfn)
        page = ""
        try:
            Server.cancel(lfn)
            page = "<div>Requested file:<p><em>%s</em></p> has been discarded from the transfer queue</div>" % lfn
        except:
            page = self.handleExc()
            pass
        page += self.getStat(user)
        page  = ajaxResponse(page)
        return page

    @expose
    def dbsStatus(self, dbs, **kwargs):
        """report status of dbs scanning"""
        cherrypy.response.headers['Content-Type'] = 'text/xml'
        try:
            idx  = self.dbs.dbslist.index(dbs)
        except:
            idx = -1
            pass
        newdbs = ""
        if  idx == len(self.dbs.dbslist)-1:
            newdbs = "no more"
        else:
            newdbs = self.dbs.dbslist[idx+1]
        page = ""
        if  idx == -1:
            page += "<img src=\"images/loading.gif\" /> please wait"
        else: 
            page += "<img src=\"images/loading.gif\" /> Scan %s instance" % dbs
            page += """<script type="text/javascript">setTimeout("ajaxdbsStatus(\'%s\')",1000)</script>""" % newdbs
        page = ajaxResponse(page,'_response')
        return page

    @expose
    def statusOne(self, user, lfn, **kwargs):
        """return status of requested LFN"""
        validate_lfn(lfn)
        cherrypy.response.headers['Content-Type'] = 'text/xml'
        lfn  = urllib.unquote(lfn)
        lfn  = lfn.strip()
        page = ""
        spanid = spanId(lfn)
        page += """<span id="%s" name="%s">""" % (spanid, spanid)
        try:
            statCode = self.setStat(user, lfn)
            if  statCode and statCode != StatusCode.TRANSFER_FAILED:
                page += "<img src=\"images/loading.gif\" /> "
            page += self.getStatForLfn(user, lfn)
            if  statCode and statCode != StatusCode.TRANSFER_FAILED:
                page += """<script type="text/javascript">setTimeout('ajaxStatusOne(\\'%s\\',\\'%s\\')',1000)</script>""" % (user, lfn)
        except:
            page = self.handleExc()
            pass
        page += "</span>"
        page = ajaxResponse(page, spanId(lfn))
        return page

    def setConfig(self, base=""):
        """configure CherryPy server"""
        mime_types = ['text/css', 'text/javascript',
                      'application/javascript', 'application/x-javascript',
                      'image/gif', 'image/png', 'image/jpg', 'image/jpeg',
                      'application/octet-stream']
        httpHeader = [('Expires',
                        time.strftime("%a, %d %b %Y %H:%M:%S GMT",
                        time.gmtime(time.time()+315360000))),
                         ('Accept-Encoding', 'gzip'),
                         ('Cache-Control', 'max-age=315360000'),
                         ('Authorization', 'Basic')
                        ]

        conf = {'/': {'tools.staticdir.root': self.fmwsdir,
                    'tools.response_headers.on':True,
                    'tools.response_headers.headers':
                      [('Expires','Thu, 15 Apr 2010 20:00:00 GMT'),
                       ('Cache-Control', 
                        'no-store, no-cache, must-revalidate,post-check=0, pre-check=0')],
                   },
              '/js'       : {'tools.gzip.on': True, 
                             'tools.gzip.mime_types':mime_types,
                             'tools.staticdir.on':True,
                             'tools.staticdir.dir':'js',
                             'tools.staticdir.content_types':{'js':'text/javascript'},
                             'tools.response_headers.on':True,
                             'tools.response_headers.headers':httpHeader
                            },
              '/css'      : {'tools.gzip.on': True, 
                             'tools.gzip.mime_types':mime_types,
                             'tools.staticdir.on':True,
                             'tools.staticdir.root': self.fmwsdir,
                             'tools.staticdir.dir':'css',
                             'tools.response_headers.on':True,
                             'tools.response_headers.headers':httpHeader
                            },
              '/download' : {
                             'tools.staticdir.on':True,
                             'tools.staticdir.root': self.download,
                             'tools.staticdir.dir':'download',
                             'tools.staticdir.content_types':{'root':'application/octet-stream'},
                             'tools.response_headers.on':True,
                             'tools.response_headers.headers':httpHeader
                            },
              '/images'   : {'tools.gzip.on': True, 
                             'tools.gzip.mime_types':mime_types,
                             'tools.staticdir.on':True,
                             'tools.staticdir.root': self.fmwsdir,
                             'tools.staticdir.dir':'images',
                             'tools.response_headers.on':True,
                             'tools.response_headers.headers':httpHeader
                            },

             }
        if  base:
            newConf = {}
            for key in conf.keys():
                newConf[key] = conf[key]
                if  key == "/":
                    newKey = "/%s" % base.replace("/","")
                else:
                    newKey = "/%s/%s" % (base.replace("/",""), key.replace("/",""))
                newConf[newKey] = conf[key]
            return newConf
        return conf


if __name__ == '__main__':

    cherrypy.response.headers['Expires'] = "Thu, 15 Apr 2010 20:00:00 GMT"
    cherrypy.response.headers['Cache-Control'] = "no-store, no-cache, must-revalidate, max-age=315360000"
    cherrypy.response.headers['Cache-Control'] = "post-check=0, pre-check=0"
    port = 8201
    cherrypy.config.update({'server.socket_port': port,
                           'server.thread_pool': 20,
                           'environment': 'production',
                           'log.screen':True,
                           'log.error_file':"fmws.log",
                           })
    fmws = FMWS()
    cherrypy.quickstart(fmws, '/', config=fmws.setConfig())
