#!/usr/bin/env python

"""
FileMover request server is based on MongoDB to keep track
of user's request to filemover.
"""

__revision__ = "$Id: RequestServer.py,v 1.3 2010/05/25 15:35:32 valya Exp $"
__version__ = "$Revision: 1.3 $"
__author__ = "Valentin Kuznetsov"

import cherrypy
import traceback
from cherrypy import expose

from pymongo.connection import Connection
from pymongo import DESCENDING

class RequestMgr(object):
    def __init__(self, dbhost='localhost', dbport=27017, limit=10):
        conn = Connection(dbhost, dbport)
        self.col = conn['filemover']['collection']
        self.limit = limit

    def allow(self, userdn):
        """Check if provided DN is allowed to make more file mover requtest"""
        spec = {'userdn': userdn}
        nres = self.col.find(spec).count()
        if  nres < self.limit:
            return "ok"
        else:
            return "fail"

    def record(self, udn, src, dst, date):
        """
        Record provided src/dst/date into filemover.collection DB
        """
        try:
            spec = {'src': src, 'dst': dst, 'date': date, 'userdn':udn}
            self.col.insert(spec)
            self.col.ensure_index([('userdn', DESCENDING)])
            return "ok"
        except:
            traceback.print_exc()
            return "fail"

class FileMoverRequestServer:
    """
    FileMover request server.
    """
    def __init__(self, dbhost='localhost', dbport=27017, limit=10):
        self.mgr = RequestMgr(dbhost, dbport, limit)

    @expose
    def default(self, **kwargs):
        """Default method"""
        return "FileMover Request Server"

    @expose
    def allow(self, userdn, **kwargs):
        """Check if provided DN is allowed to make more file mover requtest"""
        return self.mgr.allow(userdn)

    @expose
    def record(self, **kwargs):
        """
        Record provided src/dst/date into filemover.collection DB
        """
        try:
            lfn  = kwargs.get('lfn')
            src  = kwargs.get('src')
            dst  = kwargs.get('dst')
            date = kwargs.get('date')
            udn  = kwargs.get('userdn')
            return self.mgr.record(udn, src, dst, date)
        except:
            traceback.print_exc()
            return "fail"
 
if __name__ == '__main__':
    root = FileMoverRequestServer()
    cherrypy.config.update({'server.socket_port': 8202, 'server.socket_host': '0.0.0.0'})
    cherrypy.quickstart(root, '/')

