#!/usr/bin/python

import os
import sys
import tempfile
import unittest
import ConfigParser

if os.path.exists("src"):
    sys.path.append("src")
if os.path.exists('../src'):
    sys.path.append('../src')

try:
    import CmsFileServer.CmsFsTransfers
except:
    pass

class TestBlockStore(unittest.TestCase):

    def setUp(self):
        self.cp = ConfigParser.ConfigParser()

    def test_import(self):
        __import__('CmsFileServer.CmsFsTransfers')

    def test_block(self):
        self.create_block()

    def test_blockrequest(self):
        self.create_blockrequest()

    def test_partialblock(self):
        pff = CmsFileServer.CmsFsTransfers.PartialFileFactory()
        pff.configure(self.cp)
        bs = CmsFileServer.CmsFsTransfers.BlockStore(self.cp, pff)
        pf = self.create_partialfile(pff)
        self.create_partialblock(pf)

    def test_partialfilefactory(self):
        pff = CmsFileServer.CmsFsTransfers.PartialFileFactory()
        pff.configure(self.cp)

    def test_partialfile(self):
        pff = CmsFileServer.CmsFsTransfers.PartialFileFactory()
        pff.configure(self.cp)
        bs = CmsFileServer.CmsFsTransfers.BlockStore(self.cp, pff)
        self.create_partialfile(pff)

    def create_block(self):
        return CmsFileServer.CmsFsTransfers.Block(0, 100)

    def create_partialfile(self, pff):
        pf = pff.getFile('1234', '/store/blah')
        return pf

    def create_blockrequest(self):
        return CmsFileServer.CmsFsTransfers.BlockRequest(0, 100)

    def create_partialblock(self, pf):
        pb = pf.calculate_partial_block(self.create_blockrequest())
        return pb

    def configure_temporary_blockstore(self):
        tmpdir = tempfile.mkdtemp()
        if not self.cp.has_section("block_store"):
            self.cp.add_section("block_store")
        self.cp.set("block_store", "directory", tmpdir)

    def test_blockstore(self):
        pff = CmsFileServer.CmsFsTransfers.PartialFileFactory()
        pff.configure(self.cp)
        self.configure_temporary_blockstore()
        try:
            bs = CmsFileServer.CmsFsTransfers.BlockStore(self.cp, pff)
            bs.init_store()
            pf = self.create_partialfile(pff)
            partial_block = self.create_partialblock(pf)
            partial_block.write('a'*100)
            bs.store(partial_block)
            del bs
            pff = CmsFileServer.CmsFsTransfers.PartialFileFactory()
            pff.configure(self.cp)
            bs2 = CmsFileServer.CmsFsTransfers.BlockStore(self.cp, pff)
            bs2.init_store()
            pf = self.create_partialfile(pff)
            block = self.create_partialblock(pf)
            self.failUnless(isinstance(block, CmsFileServer.\
                CmsFsTransfers.Block))
            self.failUnless(bs2.retrieve(block, pf) == 'a'*100)
        finally:
            try:
                os.removedirs(self.cp.get("block_store", "directory"))
            except:
                pass

if __name__ == '__main__':
    unittest.main()
