
import os
import sys
import sets
import unittest
import ConfigParser

if os.path.exists("src"):
    sys.path.append("src")
if os.path.exists('../src'):
    sys.path.append('../src')

import CmsFileServer.DBSInteraction
from CmsFileServer.Utils import edmconfig
from CmsFileServer.CMSRunManager import CMSRunThreadWrapper

dataset = '/Cosmics/Commissioning08-CRUZET4_v1/RECO'
run = 58620
event = 151578
lumi = 6
cmsswdir = '/tmp/cmsswdir'

class TestEdmPickEvent(unittest.TestCase):

    def setUp(self):
        cp = ConfigParser.ConfigParser()
        cp.add_section("release_manager")
        cp.set("release_manager", "max_cmsrun", "2")
        cp.set("release_manager", "cmsfs_location",
            "/home/brian/tmp/gridftp_fuse")
        self.cmsrun = CMSRunThreadWrapper(cmsswdir)
        self.cmsrun.configure(cp)

    def test_event_lookup(self):
        dbs = CmsFileServer.DBSInteraction.DBS()
        fileList = dbs.getFiles(run=run, dataset=dataset, evt=event, lumi=lumi)
        releases = sets.Set()
        lfnList = [i[0] for i in fileList]
        for lfn, size in fileList:
            release = dbs.getRelease(lfn)
            releases.add(release)
        self.failUnless(len(releases) == 1, msg="Multiple releases for run.")
        try:
            os.makedirs(cmsswdir)
        except OSError, oe:
            if oe.errno != 17:
                raise
        outputfile = os.path.join(cmsswdir, 'test.root')
        config = edmconfig(release, lfnList, [(run, event, lumi)], outputfile)
        self.failUnless(config, msg="We were unable to generate the config.")
        print config
        self.cp = None
        requestid = self.cmsrun.request(cmsswdir, release, config)
        #self.cmsrun.updateparams(requestid, outputfile, requestdict)
        self.cmsrun.pool.drain()
        self.cmsrun.pool.join()
        print self.cmsrun.status(requestid)
        print self.cmsrun.process_output(requestid)
        
    def test_cms_run_manager(self):
        """
        Submit an extraction (equivalent to test_event_lookup) directly to the
        CMSRunManager.
        """
        requestdict = {'dataset'   : dataset,
                       'eventlist' : [(run, event, lumi)],
                      }
        self.cp = None
        requestId = self.cmsrun.submit(None, requestdict)
        self.cmsrun.pool.drain()
        self.cmsrun.pool.join()
        print self.cmsrun.status(requestId)
        print self.cmsrun.process_output(requestId)

    def test_cms_run_nebraska(self):
        """
        Submit an extraction for a dataset/run/event/lumi local to Nebraska
        """
        dataset = '/Cosmics/BeamCommissioning08-PromptReco-v1/RECO'
        event_ident = (62508, 11, 1)
        requestdict = {'dataset'   : dataset,
                       'eventlist' : [event_ident],
                      }
        requestId = self.cmsrun.submit(None, requestdict)
        self.cmsrun.pool.drain()
        self.cmsrun.pool.join()
        print self.cmsrun.status(requestId)
        print self.cmsrun.process_output(requestId)

    def test_cms_run_fnal(self):
        """
        Submit an extraction for a dataset/run/event/lumi local to FNAL
        """
        dataset = '/Cosmics/Commissioning08_CRAFT_V1P_doubleMuonPath_v3/RECO'
        event_ident = (66471, 252759, 8)
        requestdict = {'dataset'   : dataset,
                       'eventlist' : [event_ident],
                      }
        requestId = self.cmsrun.submit(None, requestdict)
        self.cmsrun.pool.drain()
        self.cmsrun.pool.join()
        print self.cmsrun.status(requestId)
        print self.cmsrun.process_output(requestId)

if __name__ == '__main__':
    unittest.main()

